@@ -11,7 +11,6 @@
 */
 
 #include "kiconloader.h"
-#include "kicontheme_p.h"
 
 // kdecore
 #include <KConfigGroup>
@@ -92,19 +91,6 @@ color:%7;\
 }
 }
 
-/**
- * Checks for relative paths quickly on UNIX-alikes, slowly on everything else.
- */
-static bool pathIsRelative(const QString &path)
-{
-#ifdef Q_OS_UNIX
-    // catch both /xxx/yyy and :/xxx/yyy for resources
-    return (!path.isEmpty() && path[0] != QLatin1Char('/') && path[0] != QLatin1Char(':'));
-#else
-    return QDir::isRelativePath(path);
-#endif
-}
-
 /**
  * Holds a QPixmap for this process, along with its associated path on disk.
  */
@@ -238,7 +224,7 @@ public:
     /**
      * @internal
      */
-    bool initIconThemes();
+    void initIconThemes();
 
     /**
      * @internal
@@ -307,14 +293,14 @@ public:
      * and icon state information to valid states. All parameters except the
      * name can be modified as well to be valid.
      */
-    void normalizeIconMetadata(KIconLoader::Group &group, int &size, int &state) const;
+    void normalizeIconMetadata(KIconLoader::Group &group, QSize &size, int &state) const;
 
     /**
      * @internal
      * Used with KIconLoader::loadIcon to get a base key name from the given
      * icon metadata. Ensure the metadata is normalized first.
      */
-    QString makeCacheKey(const QString &name, KIconLoader::Group group, const QStringList &overlays, int size, qreal scale, int state) const;
+    QString makeCacheKey(const QString &name, KIconLoader::Group group, const QStringList &overlays, const QSize &size, qreal scale, int state) const;
 
     /**
      * @internal
@@ -331,7 +317,7 @@ public:
      * @p size is only used for scalable images, but if non-zero non-scalable
      * images will be resized anyways.
      */
-    QImage createIconImage(const QString &path, int size = 0, qreal scale = 1.0, KIconLoader::States state = KIconLoader::DefaultState);
+    QImage createIconImage(const QString &path, const QSize &size = {}, qreal scale = 1.0, KIconLoader::States state = KIconLoader::DefaultState);
 
     /**
      * @internal
@@ -404,20 +390,6 @@ class KIconLoaderGlobalData : public QObject
 public:
     KIconLoaderGlobalData()
     {
-        // we trigger the rcc loading & fallback theme setting here instead of the old way via Q_COREAPP_STARTUP_FUNCTION
-        // so that loading the icon engine plugin doesn't trigger changing the icon theme while loading the first icon
-        // if nothing else initialized this before.
-
-        // ensure we load rcc files for application bundles (+ setup their theme)
-        initRCCIconTheme();
-
-        // Set the icon theme fallback to breeze
-        // Most of our apps use "lots" of icons that most of the times
-        // are only available with breeze, we still honour the user icon
-        // theme but if the icon is not found there, we go to breeze
-        // since it's almost sure it'll be there
-        QIcon::setFallbackThemeName(QStringLiteral("breeze"));
-
         const QStringList genericIconsFiles = QStandardPaths::locateAll(QStandardPaths::GenericDataLocation, QStringLiteral("mime/generic-icons"));
         // qCDebug(KICONTHEMES) << genericIconsFiles;
         for (const QString &file : genericIconsFiles) {
@@ -640,11 +612,10 @@ void KIconLoaderPrivate::init(const QString &_appname, const QStringList &extraS
     }
 }
 
-bool KIconLoaderPrivate::initIconThemes()
+void KIconLoaderPrivate::initIconThemes()
 {
     if (mIconThemeInited) {
-        // If mpThemeRoot isn't 0 then initing has succeeded
-        return (mpThemeRoot != nullptr);
+        return;
     }
     // qCDebug(KICONTHEMES);
     mIconThemeInited = true;
@@ -657,9 +628,9 @@ bool KIconLoaderPrivate::initIconThemes()
         qCDebug(KICONTHEMES) << "Couldn't find current icon theme, falling back to default.";
         def = new KIconTheme(KIconTheme::defaultThemeName(), appname);
         if (!def->isValid()) {
-            qWarning() << "Error: standard icon theme" << KIconTheme::defaultThemeName() << "not found!";
+            qCDebug(KICONTHEMES) << "Standard icon theme" << KIconTheme::defaultThemeName() << "not found!";
             delete def;
-            return false;
+            return;
         }
     }
     mpThemeRoot = new KIconThemeNode(def);
@@ -674,8 +645,6 @@ bool KIconLoaderPrivate::initIconThemes()
     searchPaths.append(QStringLiteral("icons")); // was xdgdata-icon in KStandardDirs
     // These are not in the icon spec, but e.g. GNOME puts some icons there anyway.
     searchPaths.append(QStringLiteral("pixmaps")); // was xdgdata-pixmaps in KStandardDirs
-
-    return true;
 }
 
 KIconLoader::~KIconLoader() = default;
@@ -832,15 +801,15 @@ QString KIconLoaderPrivate::removeIconExtension(const QString &name) const
     return name;
 }
 
-void KIconLoaderPrivate::normalizeIconMetadata(KIconLoader::Group &group, int &size, int &state) const
+void KIconLoaderPrivate::normalizeIconMetadata(KIconLoader::Group &group, QSize &size, int &state) const
 {
     if ((state < 0) || (state >= KIconLoader::LastState)) {
         qWarning() << "Illegal icon state:" << state;
         state = KIconLoader::DefaultState;
     }
 
-    if (size < 0) {
-        size = 0;
+    if (size.width() < 0 || size.height() < 0) {
+        size = {};
     }
 
     // For "User" icons, bail early since the size should be based on the size on disk,
@@ -855,16 +824,16 @@ void KIconLoaderPrivate::normalizeIconMetadata(KIconLoader::Group &group, int &s
     }
 
     // If size == 0, use default size for the specified group.
-    if (size == 0) {
+    if (size.isNull()) {
         if (group < 0) {
             qWarning() << "Neither size nor group specified!";
             group = KIconLoader::Desktop;
         }
-        size = mpGroups[group].size;
+        size = QSize(mpGroups[group].size, mpGroups[group].size);
     }
 }
 
-QString KIconLoaderPrivate::makeCacheKey(const QString &name, KIconLoader::Group group, const QStringList &overlays, int size, qreal scale, int state) const
+QString KIconLoaderPrivate::makeCacheKey(const QString &name, KIconLoader::Group group, const QStringList &overlays, const QSize &size, qreal scale, int state) const
 {
     // The KSharedDataCache is shared so add some namespacing. The following code
     // uses QStringBuilder (new in Qt 4.6)
@@ -873,7 +842,7 @@ QString KIconLoaderPrivate::makeCacheKey(const QString &name, KIconLoader::Group
     return (group == KIconLoader::User ? QLatin1String("$kicou_") : QLatin1String("$kico_"))
             % name
             % QLatin1Char('_')
-            % QString::number(size)
+            % (size.width() == size.height() ? QString::number(size.height()) : QString::number(size.height()) % QLatin1Char('x') % QString::number(size.width()))
             % QLatin1Char('@')
             % QString::number(scale, 'f', 1)
             % QLatin1Char('_')
@@ -939,13 +908,13 @@ QByteArray KIconLoaderPrivate::processSvg(const QString &path, KIconLoader::Stat
     return processedContents;
 }
 
-QImage KIconLoaderPrivate::createIconImage(const QString &path, int size, qreal scale, KIconLoader::States state)
+QImage KIconLoaderPrivate::createIconImage(const QString &path, const QSize &size, qreal scale, KIconLoader::States state)
 {
     // TODO: metadata in the theme to make it do this only if explicitly supported?
     QImageReader reader;
     QBuffer buffer;
 
-    if (q->theme()->followsColorScheme() && (path.endsWith(QLatin1String("svg")) || path.endsWith(QLatin1String("svgz")))) {
+    if (q->theme() && q->theme()->followsColorScheme() && (path.endsWith(QLatin1String("svg")) || path.endsWith(QLatin1String("svgz")))) {
         buffer.setData(processSvg(path, state));
         reader.setDevice(&buffer);
     } else {
@@ -956,8 +925,18 @@ QImage KIconLoaderPrivate::createIconImage(const QString &path, int size, qreal
         return QImage();
     }
 
-    if (size != 0) {
-        reader.setScaledSize(QSize(size * scale, size * scale));
+    if (!size.isNull()) {
+        // ensure we keep aspect ratio
+        const QSize wantedSize = size * scale;
+        QSize finalSize(reader.size());
+        if (finalSize.isNull()) {
+            // nothing to scale
+            finalSize = wantedSize;
+        } else {
+            // like QSvgIconEngine::pixmap try to keep aspect ratio
+            finalSize.scale(wantedSize, Qt::KeepAspectRatio);
+        }
+        reader.setScaledSize(finalSize);
     }
 
     return reader.read();
@@ -1200,15 +1179,14 @@ QString KIconLoader::iconPath(const QString &_name, int group_or_size, bool canR
 
 QString KIconLoader::iconPath(const QString &_name, int group_or_size, bool canReturnNull, qreal scale) const
 {
-    if (!d->initIconThemes()) {
-        return QString();
-    }
-
-    if (_name.isEmpty() || !pathIsRelative(_name)) {
+    // we need to honor resource :/ paths and QDir::searchPaths => use QDir::isAbsolutePath, see bug 434451
+    if (_name.isEmpty() || QDir::isAbsolutePath(_name)) {
         // we have either an absolute path or nothing to work with
         return _name;
     }
 
+    d->initIconThemes();
+
     QString name = d->removeIconExtension(_name);
 
     QString path;
@@ -1303,14 +1281,29 @@ QPixmap KIconLoader::loadScaledIcon(const QString &_name,
                                     const QStringList &overlays,
                                     QString *path_store,
                                     bool canReturnNull) const
+{
+    return loadScaledIcon(_name, group, scale, QSize(size, size), state, overlays, path_store, canReturnNull);
+}
+
+QPixmap KIconLoader::loadScaledIcon(const QString &_name,
+                                    KIconLoader::Group group,
+                                    qreal scale,
+                                    const QSize &_size,
+                                    int state,
+                                    const QStringList &overlays,
+                                    QString *path_store,
+                                    bool canReturnNull) const
+
 {
     QString name = _name;
     bool favIconOverlay = false;
 
-    if (size < 0 || _name.isEmpty()) {
+    if (_size.width() < 0 || _size.height() < 0 || _name.isEmpty()) {
         return QPixmap();
     }
 
+    QSize size = _size;
+
     /*
      * This method works in a kind of pipeline, with the following steps:
      * 1. Sanity checks.
@@ -1327,7 +1320,8 @@ QPixmap KIconLoader::loadScaledIcon(const QString &_name,
         name = QStandardPaths::writableLocation(QStandardPaths::GenericCacheLocation) + QLatin1Char('/') + name + QStringLiteral(".png");
     }
 
-    bool absolutePath = !pathIsRelative(name);
+    // we need to honor resource :/ paths and QDir::searchPaths => use QDir::isAbsolutePath, see bug 434451
+    const bool absolutePath = QDir::isAbsolutePath(name);
     if (!absolutePath) {
         name = d->removeIconExtension(name);
     }
@@ -1367,11 +1361,9 @@ QPixmap KIconLoader::loadScaledIcon(const QString &_name,
     }
 
     // Image is not cached... go find it and apply effects.
-    if (!d->initIconThemes()) {
-        return QPixmap();
-    }
+    d->initIconThemes();
 
-    favIconOverlay = favIconOverlay && size > 22;
+    favIconOverlay = favIconOverlay && std::min(size.height(), size.width()) > 22;
 
     // First we look for non-User icons. If we don't find one we'd search in
     // the User space anyways...
@@ -1379,7 +1371,7 @@ QPixmap KIconLoader::loadScaledIcon(const QString &_name,
         if (absolutePath && !favIconOverlay) {
             path = name;
         } else {
-            path = d->findMatchingIconWithGenericFallbacks(favIconOverlay ? QStringLiteral("text-html") : name, size, scale);
+            path = d->findMatchingIconWithGenericFallbacks(favIconOverlay ? QStringLiteral("text-html") : name, std::min(size.height(), size.width()), scale);
         }
     }
 
@@ -1391,7 +1383,7 @@ QPixmap KIconLoader::loadScaledIcon(const QString &_name,
     // Still can't find it? Use "unknown" if we can't return null.
     // We keep going in the function so we can ensure this result gets cached.
     if (path.isEmpty() && !canReturnNull) {
-        path = d->unknownIconPath(size, scale);
+        path = d->unknownIconPath(std::min(size.height(), size.width()), scale);
         iconWasUnknown = true;
     }
 
